# Copyright 2024 The Kubeflow Authors
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from typing import Dict

from google.protobuf import json_format
from kfp.dsl import PipelineTask
from kfp.kubernetes import common
from kfp.kubernetes import kubernetes_executor_config_pb2 as pb


def use_config_map_as_env(
    task: PipelineTask,
    config_map_name: str,
    config_map_key_to_env: Dict[str, str],
) -> PipelineTask:
    """Use a Kubernetes ConfigMap as an environment variable as described by the `Kubernetes documentation
    https://kubernetes.io/docs/tasks/configure-pod-container/configure-pod-configmap/#define-container-environment-variables-using-configmap-data` _.

    Args:
        task: Pipeline task.
        config_map_name: Name of the ConfigMap.
        config_map_key_to_env: Dictionary of ConfigMap key to environment variable name. For example, ``{'foo': 'FOO'}`` sets the value of the ConfigMap's foo field to the environment variable ``FOO``.

    Returns:
        Task object with updated ConfigMap configuration.
    """

    msg = common.get_existing_kubernetes_config_as_message(task)

    key_to_env = [
        pb.ConfigMapAsEnv.ConfigMapKeyToEnvMap(
            config_map_key=config_map_key,
            env_var=env_var,
        ) for config_map_key, env_var in config_map_key_to_env.items()
    ]
    config_map_as_env = pb.ConfigMapAsEnv(
        config_map_name=config_map_name,
        key_to_env=key_to_env,
    )

    msg.config_map_as_env.append(config_map_as_env)

    task.platform_config['kubernetes'] = json_format.MessageToDict(msg)

    return task


def use_config_map_as_volume(
    task: PipelineTask,
    config_map_name: str,
    mount_path: str,
    optional: bool = False,
) -> PipelineTask:
    """Use a Kubernetes ConfigMap by mounting its data to the task's container as
    described by the `Kubernetes documentation <https://kubernetes.io/docs/tasks/configure-pod-container/configure-pod-configmap/#add-configmap-data-to-a-volume>`_.

    Args:
        task: Pipeline task.
        config_map_name: Name of the ConfigMap.
        mount_path: Path to which to mount the ConfigMap data.
        optional: Optional field specifying whether the ConfigMap must be defined.

    Returns:
        Task object with updated ConfigMap configuration.
    """

    msg = common.get_existing_kubernetes_config_as_message(task)

    config_map_as_vol = pb.ConfigMapAsVolume(
        config_map_name=config_map_name,
        mount_path=mount_path,
        optional=optional,
    )
    msg.config_map_as_volume.append(config_map_as_vol)

    task.platform_config['kubernetes'] = json_format.MessageToDict(msg)

    return task



# Copyright 2023 The Kubeflow Authors
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import os
import subprocess
import sys

try:
    from distutils.spawn import find_executable
except ImportError:
    from shutil import which as find_executable

PLATFORM_DIR = os.path.realpath(os.path.dirname(os.path.dirname(__file__)))

PROTO_DIR = os.path.join(PLATFORM_DIR, 'proto')

PKG_DIR = os.path.realpath(
    os.path.join(PLATFORM_DIR, 'python', 'kfp', 'kubernetes'))

# Find the Protocol Compiler. (Taken from protobuf/python/setup.py)
if 'PROTOC' in os.environ and os.path.exists(os.environ['PROTOC']):
    PROTOC = os.environ['PROTOC']
else:
    PROTOC = find_executable('protoc')


def generate_proto(source: str) -> None:
    """Generate a _pb2.py from a .proto file.

    Invokes the Protocol Compiler to generate a _pb2.py from the given
    .proto file.  Does nothing if the output already exists and is newer than
    the input.

    Args:
      source: The source proto file that needs to be compiled.
    """
    output = source.replace('.proto', '_pb2.py')

    if not os.path.exists(output) or (
            os.path.exists(source) and
            os.path.getmtime(source) > os.path.getmtime(output)):
        print(f'Generating {output}...')

        if not os.path.exists(source):
            sys.stderr.write(f"Can't find required file: {source}\n")
            sys.exit(-1)

        if PROTOC is None:
            sys.stderr.write(
                'protoc is not found. Please compile it or install the binary package.\n'
            )
            sys.exit(-1)

        protoc_command = [
            PROTOC,
            f'-I={PROTO_DIR}',
            f'--experimental_allow_proto3_optional',
            f'--python_out={PKG_DIR}',
            source,
        ]

        if subprocess.call(protoc_command) != 0:
            sys.exit(-1)


if __name__ == '__main__':
    generate_proto(os.path.join(PROTO_DIR, 'kubernetes_executor_config.proto'))



# Copyright 2024 The Kubeflow Authors
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from google.protobuf import json_format
from kfp.dsl import PipelineTask
from kfp.kubernetes import common


def add_pod_label(
    task: PipelineTask,
    label_key: str,
    label_value: str,
) -> PipelineTask:
    """Add a label to the task Pod's `metadata
    <https://kubernetes.io/docs/reference/kubernetes-api/workload-resources/pod-v1/#Pod>`_.

    Each label is a key-value pair, corresponding to the metadata's `ObjectMeta <https://kubernetes.io/docs/reference/kubernetes-api/common-definitions/object-meta/#ObjectMeta`_ field.

    Args:
        task: Pipeline task.
        label_key: Key of the metadata label.
        label_value: Value of the metadata label.

    Returns:
        Task object with an added metadata label.
    """

    msg = common.get_existing_kubernetes_config_as_message(task)
    msg.pod_metadata.labels.update({label_key: label_value})
    task.platform_config['kubernetes'] = json_format.MessageToDict(msg)

    return task


def add_pod_annotation(
    task: PipelineTask,
    annotation_key: str,
    annotation_value: str,
) -> PipelineTask:
    """Add an annotation to the task Pod's `metadata
    <https://kubernetes.io/docs/reference/kubernetes-api/workload-resources/pod-v1/#Pod>`_.

    Each annotation is a key-value pair, corresponding to the metadata's `ObjectMeta <https://kubernetes.io/docs/reference/kubernetes-api/common-definitions/object-meta/#ObjectMeta`_ field.

    Args:
        task: Pipeline task.
        annotation_key: Key of the metadata annotation.
        annotation_value: Value of the metadata annotation.

    Returns:
        Task object with an added metadata annotation.
    """

    msg = common.get_existing_kubernetes_config_as_message(task)
    msg.pod_metadata.annotations.update({annotation_key: annotation_value})
    task.platform_config['kubernetes'] = json_format.MessageToDict(msg)

    return task



# Copyright 2023 The Kubeflow Authors
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

__version__ = '1.4.0'

__all__ = [
    'add_ephemeral_volume',
    'add_node_selector',
    'add_pod_annotation',
    'add_pod_label',
    'add_toleration',
    'CreatePVC',
    'DeletePVC',
    'empty_dir_mount',
    'mount_pvc',
    'set_image_pull_policy',
    'use_field_path_as_env',
    'set_image_pull_secrets',
    'set_timeout',
    'use_config_map_as_env',
    'use_config_map_as_volume',
    'use_secret_as_env',
    'use_secret_as_volume',
]

from kfp.kubernetes.config_map import use_config_map_as_env
from kfp.kubernetes.config_map import use_config_map_as_volume
from kfp.kubernetes.empty_dir import empty_dir_mount
from kfp.kubernetes.field import use_field_path_as_env
from kfp.kubernetes.image import set_image_pull_policy
from kfp.kubernetes.image import set_image_pull_secrets
from kfp.kubernetes.node_selector import add_node_selector
from kfp.kubernetes.pod_metadata import add_pod_annotation
from kfp.kubernetes.pod_metadata import add_pod_label
from kfp.kubernetes.secret import use_secret_as_env
from kfp.kubernetes.secret import use_secret_as_volume
from kfp.kubernetes.timeout import set_timeout
from kfp.kubernetes.toleration import add_toleration
from kfp.kubernetes.volume import add_ephemeral_volume
from kfp.kubernetes.volume import CreatePVC
from kfp.kubernetes.volume import DeletePVC
from kfp.kubernetes.volume import mount_pvc



# Copyright 2023 The Kubeflow Authors
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

# Configuration file for the Sphinx documentation builder.
#
# This file does only contain a selection of the most common options. For a
# full list see the documentation:
# http://www.sphinx-doc.org/en/master/config

import re

from kfp import dsl


# preserve function docstrings for components by setting component decorators to passthrough decorators
# also enables autodoc to document the components as functions without using the autodata directive (https://www.sphinx-doc.org/en/master/usage/extensions/autodoc.html#directive-autodata)
def container_component_decorator(func):
    return func


def component_decorator(*args, **kwargs):

    def decorator(func):
        return func

    return decorator


dsl.component = component_decorator
dsl.container_component = container_component_decorator

# -- Project information -----------------------------------------------------
project = 'Kubeflow Pipelines'
copyright = '2023, The Kubeflow Authors'
author = 'The Kubeflow Authors'

# The short X.Y version
version = ''
# The full version, including alpha/beta/rc tags
release = ''

# -- General configuration ---------------------------------------------------

# If your documentation needs a minimal Sphinx version, state it here.
# needs_sphinx = '1.0'

# Add any Sphinx extension module names here, as strings. They can be
# extensions coming with Sphinx (named 'sphinx.ext.*') or your custom
# ones.
extensions = [
    'sphinx.ext.autodoc',
    'sphinx.ext.viewcode',
    'sphinx.ext.napoleon',
    'sphinx_click',
    'm2r2',
    'sphinx_immaterial',
    'autodocsumm',
]
autodoc_member_order = 'bysource'
autodoc_default_options = {
    'members': True,
    'imported-members': True,
    'undoc-members': True,
    'show-inheritance': False,
    'autosummary': True,
}

html_theme = 'sphinx_immaterial'
html_title = 'KFP SDK kfp-kubernetes API Reference'
html_static_path = ['_static']
html_css_files = ['custom.css']
html_logo = '_static/kubeflow.png'
html_favicon = '_static/favicon.ico'
html_theme_options = {
    'icon': {
        'repo': 'fontawesome/brands/github',
    },
    'repo_url':
        'https://github.com/kubeflow/pipelines/tree/master/kubernetes_platform',
    'repo_name':
        'pipelines',
    'repo_type':
        'github',
    'edit_uri':
        'blob/master/docs',
    'globaltoc_collapse':
        False,
    'features': [
        'navigation.expand',
        # "navigation.tabs",
        # "toc.integrate",
        'navigation.sections',
        # "navigation.instant",
        # "header.autohide",
        'navigation.top',
        # "navigation.tracking",
        'search.highlight',
        'search.share',
        'toc.follow',
        'toc.sticky',
    ],
    'palette': [
        {
            'media': '(prefers-color-scheme: dark)',
            'scheme': 'slate',
            'primary': 'kfpblue',
            # "accent": "lime",
            'toggle': {
                'icon': 'material/lightbulb',
                'name': 'Switch to light mode',
            },
        },
        {
            'media': '(prefers-color-scheme: light)',
            'scheme': 'default',
            'primary': 'kfpblue',
            # "accent": "light-blue",
            'toggle': {
                'icon': 'material/lightbulb-outline',
                'name': 'Switch to dark mode',
            },
        },
    ],
    'font': {
        'text': 'Open Sans'
    },
    'version_dropdown':
        True,
    'version_info': [
        {
            'version':
                'https://kfp-kubernetes.readthedocs.io/en/kfp-kubernetes-1.4.0/',
            'title':
                '1.4.0',
            'aliases': ['stable'],
        },
        {
            'version':
                'https://kfp-kubernetes.readthedocs.io/en/kfp-kubernetes-1.3.0/',
            'title':
                '1.3.0',
            'aliases': [],
        },
        {
            'version':
                'https://kfp-kubernetes.readthedocs.io/en/kfp-kubernetes-1.2.0/',
            'title':
                '1.2.0',
            'aliases': [],
        },
        {
            'version':
                'https://kfp-kubernetes.readthedocs.io/en/kfp-kubernetes-1.1.0/',
            'title':
                '1.1.0',
            'aliases': [],
        },
        {
            'version':
                'https://kfp-kubernetes.readthedocs.io/en/kfp-kubernetes-1.0.0/',
            'title':
                '1.0.0',
            'aliases': [],
        },
        {
            'version':
                'https://kfp-kubernetes.readthedocs.io/en/kfp-kubernetes-0.0.1/',
            'title':
                '0.0.1',
            'aliases': [],
        },
    ],
    # "toc_title_is_page_title": True,
}
# Add any paths that contain templates here, relative to this directory.
templates_path = ['_templates']

# The suffix(es) of source filenames.
# You can specify multiple suffix as a list of string:
source_suffix = '.rst'

# The master toctree document.
master_doc = 'index'

# The language for content autogenerated by Sphinx. Refer to documentation
# for a list of supported languages.
#
# This is also used if you do content translation via gettext catalogs.
# Usually you set "language" from the command line for these cases.
language = 'en'

# List of patterns, relative to source directory, that match files and
# directories to ignore when looking for source files.
# This pattern also affects html_static_path and html_extra_path.
exclude_patterns = ['_build', 'Thumbs.db', '.DS_Store']

# The name of the Pygments (syntax highlighting) style to use.
pygments_style = None

# The default sidebars (for documents that don't match any pattern) are
# defined by theme itself.  Builtin themes are using these templates by
# default: ``['localtoc.html', 'relations.html', 'sourcelink.html',
# 'searchbox.html']``.
#
# html_sidebars = {}

# -- Options for HTMLHelp output ---------------------------------------------

# Output file base name for HTML help builder.
htmlhelp_basename = 'KfpKubernetesdoc'


# TODO: align with GCPC representation of components (in particular, OutputPath and Output[])
def strip_outputs_from_signature(app, what, name, obj, options, signature,
                                 return_annotation):
    if signature is not None:
        signature = re.sub(
            r'[0-9a-zA-Z]+: <kfp\.components\.types\.type_annotations\.OutputPath object at 0x[0-9a-fA-F]+>?,?\s',
            '', signature)
    return signature, return_annotation


def setup(app):
    app.connect('autodoc-process-signature', strip_outputs_from_signature)



# Copyright 2024 The Kubeflow Authors
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from google.protobuf import json_format
from kfp import dsl
from kfp import kubernetes


class TestEmptyDirMounts:

    def test_add_one(self):

        @dsl.pipeline
        def my_pipeline():
            task = comp()
            kubernetes.empty_dir_mount(
                task, 
                volume_name='emptydir-vol-1', 
                mount_path='/mnt/my_vol_1',
                medium='Memory',
                size_limit='1Gi'
            )

        assert json_format.MessageToDict(my_pipeline.platform_spec) == {
            'platforms': {
                'kubernetes': {
                    'deploymentSpec': {
                        'executors': {
                            'exec-comp': {
                                'emptyDirMounts': [{
                                    'medium': 'Memory',
                                    'mountPath': '/mnt/my_vol_1',
                                    'sizeLimit': '1Gi',
                                    'volumeName': 'emptydir-vol-1'
                                }]
                            }
                        }
                    }
                }
            }
        }

    def test_add_two(self):

        @dsl.pipeline
        def my_pipeline():
            task = comp()
            kubernetes.empty_dir_mount(
                task, 
                volume_name='emptydir-vol-1',
                mount_path='/mnt/my_vol_1',
                medium='Memory',
                size_limit='1Gi'
            )
            kubernetes.empty_dir_mount(
                task, 
                volume_name='emptydir-vol-2',
                mount_path='/mnt/my_vol_2'
            )

        assert json_format.MessageToDict(my_pipeline.platform_spec) == {
            'platforms': {
                'kubernetes': {
                    'deploymentSpec': {
                        'executors': {
                            'exec-comp': {
                                'emptyDirMounts': [{
                                    'medium': 'Memory',
                                    'mountPath': '/mnt/my_vol_1',
                                    'sizeLimit': '1Gi',
                                    'volumeName': 'emptydir-vol-1'
                                },
                                {
                                    'mountPath': '/mnt/my_vol_2',
                                    'volumeName': 'emptydir-vol-2'
                                }]
                            }
                        }
                    }
                }
            }
        }

    def test_respects_other_configuration(self):

        @dsl.pipeline
        def my_pipeline():
            task = comp()

            kubernetes.empty_dir_mount(
                task, 
                volume_name='emptydir-vol-1',
                mount_path='/mnt/my_vol_1',
                medium='Memory',
                size_limit='1Gi'
            )

            # this should exist too
            kubernetes.set_image_pull_secrets(task, ['secret-name'])

        assert json_format.MessageToDict(my_pipeline.platform_spec) == {
            'platforms': {
                'kubernetes': {
                    'deploymentSpec': {
                        'executors': {
                            'exec-comp': {
                                'emptyDirMounts': [{
                                    'medium': 'Memory',
                                    'mountPath': '/mnt/my_vol_1',
                                    'sizeLimit': '1Gi',
                                    'volumeName': 'emptydir-vol-1'
                                }],
                                'imagePullSecret': [{
                                    'secretName': 'secret-name'
                                }]
                            }
                        }
                    }
                }
            }
        }

@dsl.component
def comp():
    pass



# Copyright 2024 The Kubeflow Authors
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from google.protobuf import json_format
from kfp import dsl
from kfp import kubernetes


class TestUseFieldPathAsEnv:

    def test_use_one(self):

        @dsl.pipeline
        def my_pipeline():
            task = comp()
            kubernetes.use_field_path_as_env(
                task,
                env_name="KFP_RUN_NAME",
                field_path="metadata.annotations['pipelines.kubeflow.org/run_name']"
            )

        assert json_format.MessageToDict(my_pipeline.platform_spec) == {
            'platforms': {
                'kubernetes': {
                    'deploymentSpec': {
                        'executors': {
                            'exec-comp': {
                                'fieldPathAsEnv': [{
                                    'name':
                                        'KFP_RUN_NAME',
                                    'fieldPath':
                                        'metadata.annotations[\'pipelines.kubeflow.org/run_name\']'
                                }]
                            }
                        }
                    }
                }
            }
        }

    def test_use_two(self):

        @dsl.pipeline
        def my_pipeline():
            task = comp()
            kubernetes.use_field_path_as_env(
                task,
                env_name="KFP_RUN_NAME",
                field_path="metadata.annotations['pipelines.kubeflow.org/run_name']"
            )
            kubernetes.use_field_path_as_env(
                task,
                env_name="POD_NAME",
                field_path="metadata.name"
            )

        assert json_format.MessageToDict(my_pipeline.platform_spec) == {
            'platforms': {
                'kubernetes': {
                    'deploymentSpec': {
                        'executors': {
                            'exec-comp': {
                                'fieldPathAsEnv': [{
                                    'name':
                                        'KFP_RUN_NAME',
                                    'fieldPath':
                                        'metadata.annotations[\'pipelines.kubeflow.org/run_name\']'
                                },
                                {
                                    'name':
                                        'POD_NAME',
                                    'fieldPath':
                                        'metadata.name'
                                }]
                            }
                        }
                    }
                }
            }
        }


@dsl.component
def comp():
    pass



# Copyright 2023 The Kubeflow Authors
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from google.protobuf import json_format
from kfp import dsl
from kfp import kubernetes
import pytest


class TestMountPVC:

    def test_mount_one(self):

        @dsl.pipeline
        def my_pipeline():
            task = comp()
            kubernetes.mount_pvc(
                task,
                pvc_name='pvc-name',
                mount_path='path',
            )

        assert json_format.MessageToDict(my_pipeline.platform_spec) == {
            'platforms': {
                'kubernetes': {
                    'deploymentSpec': {
                        'executors': {
                            'exec-comp': {
                                'pvcMount': [{
                                    'constant': 'pvc-name',
                                    'mountPath': 'path'
                                }]
                            }
                        }
                    }
                }
            }
        }

    def test_mount_two(self):

        @dsl.pipeline
        def my_pipeline():
            task = comp()
            kubernetes.mount_pvc(
                task,
                pvc_name='pvc-name',
                mount_path='path1',
            )
            kubernetes.mount_pvc(
                task,
                pvc_name='other-pvc-name',
                mount_path='path2',
            )

        assert json_format.MessageToDict(my_pipeline.platform_spec) == {
            'platforms': {
                'kubernetes': {
                    'deploymentSpec': {
                        'executors': {
                            'exec-comp': {
                                'pvcMount': [
                                    {
                                        'constant': 'pvc-name',
                                        'mountPath': 'path1'
                                    },
                                    {
                                        'constant': 'other-pvc-name',
                                        'mountPath': 'path2'
                                    },
                                ]
                            }
                        }
                    }
                }
            }
        }

    def test_mount_preserves_secret_as_env(self):
        # checks that mount_pvc respects previously set secrets
        @dsl.pipeline
        def my_pipeline():
            task = comp()
            kubernetes.use_secret_as_env(
                task,
                secret_name='secret-name',
                secret_key_to_env={'password': 'SECRET_VAR'},
            )
            kubernetes.mount_pvc(
                task,
                pvc_name='pvc-name',
                mount_path='path',
            )

        assert json_format.MessageToDict(my_pipeline.platform_spec) == {
            'platforms': {
                'kubernetes': {
                    'deploymentSpec': {
                        'executors': {
                            'exec-comp': {
                                'pvcMount': [{
                                    'constant': 'pvc-name',
                                    'mountPath': 'path'
                                }],
                                'secretAsEnv': [{
                                    'secretName':
                                        'secret-name',
                                    'keyToEnv': [{
                                        'secretKey': 'password',
                                        'envVar': 'SECRET_VAR'
                                    }]
                                }]
                            }
                        }
                    }
                }
            }
        }

    def test_mount_preserves_secret_as_vol(self):
        # checks that mount_pvc respects previously set secrets
        @dsl.pipeline
        def my_pipeline():
            task = comp()
            kubernetes.use_secret_as_volume(
                task,
                secret_name='secret-name',
                mount_path='secretpath',
            )
            kubernetes.mount_pvc(
                task,
                pvc_name='pvc-name',
                mount_path='path',
            )

        assert json_format.MessageToDict(my_pipeline.platform_spec) == {
            'platforms': {
                'kubernetes': {
                    'deploymentSpec': {
                        'executors': {
                            'exec-comp': {
                                'pvcMount': [{
                                    'constant': 'pvc-name',
                                    'mountPath': 'path'
                                }],
                                'secretAsVolume': [{
                                    'secretName': 'secret-name',
                                    'mountPath': 'secretpath',
                                    'optional': False
                                }]
                            }
                        }
                    }
                }
            }
        }

    def test_illegal_pvc_name(self):

        @dsl.component
        def identity(string: str) -> str:
            return string

        with pytest.raises(
                ValueError,
                match=r'Argument for \'pvc_name\' must be an instance of str or PipelineChannel\. Got unknown input type: <class \'int\'>\.',
        ):

            @dsl.pipeline
            def my_pipeline(string: str = 'string'):
                op1 = kubernetes.mount_pvc(
                    identity(string=string),
                    pvc_name=1,
                    mount_path='/path',
                )


class TestGenericEphemeralVolume:

    def test_mount_one(self):

        @dsl.pipeline
        def my_pipeline():
            task = comp()
            kubernetes.add_ephemeral_volume(
                task,
                volume_name='pvc-name',
                mount_path='path',
                access_modes=['ReadWriteOnce'],
                size='5Gi',
            )

        assert json_format.MessageToDict(my_pipeline.platform_spec) == {
            'platforms': {
                'kubernetes': {
                    'deploymentSpec': {
                        'executors': {
                            'exec-comp': {
                                'genericEphemeralVolume': [{
                                    'volumeName': 'pvc-name',
                                    'mountPath': 'path',
                                    'accessModes': ['ReadWriteOnce'],
                                    'defaultStorageClass': True,
                                    'size': '5Gi',
                                }]
                            }
                        }
                    }
                }
            }
        }

    def test_mount_two(self):

        @dsl.pipeline
        def my_pipeline():
            task = comp()
            kubernetes.add_ephemeral_volume(
                task,
                volume_name='pvc-name',
                mount_path='path1',
                access_modes=['ReadWriteOnce'],
                size='5Gi',
            )
            kubernetes.add_ephemeral_volume(
                task,
                volume_name='other-pvc-name',
                mount_path='path2',
                access_modes=['ReadWriteMany'],
                size='10Ti',
                storage_class_name='gp2',
                labels={
                    'label1': 'l1',
                },
                annotations={
                    'annotation1': 'a1',
                }
            )

        assert json_format.MessageToDict(my_pipeline.platform_spec) == {
            'platforms': {
                'kubernetes': {
                    'deploymentSpec': {
                        'executors': {
                            'exec-comp': {
                                'genericEphemeralVolume': [
                                    {
                                        'volumeName': 'pvc-name',
                                        'mountPath': 'path1',
                                        'accessModes': ['ReadWriteOnce'],
                                        'defaultStorageClass': True,
                                        'size': '5Gi',
                                    },
                                    {
                                        'volumeName': 'other-pvc-name',
                                        'mountPath': 'path2',
                                        'accessModes': ['ReadWriteMany'],
                                        'size': '10Ti',
                                        'storageClassName': 'gp2',
                                        'metadata': {
                                            'labels': {'label1': 'l1'},
                                            'annotations': {'annotation1': 'a1'},
                                        },
                                    },
                                ]
                            }
                        }
                    }
                }
            }
        }


@dsl.component
def comp():
    pass



# Copyright 2024 The Kubeflow Authors
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from google.protobuf import json_format
from kfp import dsl
from kfp import kubernetes


class TestImagePullPolicy:

    def test_always(self):

        @dsl.pipeline
        def my_pipeline():
            task = comp()
            kubernetes.set_image_pull_policy(task, 'Always')

        assert json_format.MessageToDict(my_pipeline.platform_spec) == {
            'platforms': {
                'kubernetes': {
                    'deploymentSpec': {
                        'executors': {
                            'exec-comp': {
                                'imagePullPolicy': 'Always'
                            }
                        }
                    }
                }
            }
        }

    def test_if_not_present(self):

        @dsl.pipeline
        def my_pipeline():
            task = comp()
            kubernetes.set_image_pull_policy(task, 'IfNotPresent')

        assert json_format.MessageToDict(my_pipeline.platform_spec) == {
            'platforms': {
                'kubernetes': {
                    'deploymentSpec': {
                        'executors': {
                            'exec-comp': {
                                'imagePullPolicy': 'IfNotPresent'
                            }
                        }
                    }
                }
            }
        }

    def test_never(self):

        @dsl.pipeline
        def my_pipeline():
            task = comp()
            kubernetes.set_image_pull_policy(task, 'Never')

        assert json_format.MessageToDict(my_pipeline.platform_spec) == {
            'platforms': {
                'kubernetes': {
                    'deploymentSpec': {
                        'executors': {
                            'exec-comp': {
                                'imagePullPolicy': 'Never'
                            }
                        }
                    }
                }
            }
        }


@dsl.component
def comp():
    pass



# Copyright 2024 The Kubeflow Authors
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from google.protobuf import json_format
from kfp import compiler
from kfp import dsl
from kfp import kubernetes


class TestTolerations:

    def test_add_one(self):

        @dsl.pipeline
        def my_pipeline():
            task = comp()
            kubernetes.add_toleration(
                task,
                key='key1',
                operator='Equal',
                value='value1',
                effect='NoSchedule',
            )

        compiler.Compiler().compile(
            pipeline_func=my_pipeline, package_path='my_pipeline.yaml')

        assert json_format.MessageToDict(my_pipeline.platform_spec) == {
            'platforms': {
                'kubernetes': {
                    'deploymentSpec': {
                        'executors': {
                            'exec-comp': {
                                'tolerations': [{
                                    'key': 'key1',
                                    'operator': 'Equal',
                                    'value': 'value1',
                                    'effect': 'NoSchedule',
                                }]
                            }
                        }
                    }
                }
            }
        }

    def test_add_one_with_toleration_seconds(self):

        @dsl.pipeline
        def my_pipeline():
            task = comp()
            kubernetes.add_toleration(
                task,
                key='key1',
                operator='Equal',
                value='value1',
                effect='NoExecute',
                toleration_seconds=10,
            )

        compiler.Compiler().compile(
            pipeline_func=my_pipeline, package_path='my_pipeline.yaml')

        assert json_format.MessageToDict(my_pipeline.platform_spec) == {
            'platforms': {
                'kubernetes': {
                    'deploymentSpec': {
                        'executors': {
                            'exec-comp': {
                                'tolerations': [{
                                    'key': 'key1',
                                    'operator': 'Equal',
                                    'value': 'value1',
                                    'effect': 'NoExecute',
                                    'tolerationSeconds': '10',
                                }]
                            }
                        }
                    }
                }
            }
        }

    def test_add_two(self):

        @dsl.pipeline
        def my_pipeline():
            task = comp()
            kubernetes.add_toleration(
                task,
                key='key1',
                operator='Equal',
                value='value1',
            )
            kubernetes.add_toleration(
                task,
                key='key2',
                operator='Equal',
                value='value2',
            )

        assert json_format.MessageToDict(my_pipeline.platform_spec) == {
            'platforms': {
                'kubernetes': {
                    'deploymentSpec': {
                        'executors': {
                            'exec-comp': {
                                'tolerations': [
                                    {
                                        'key': 'key1',
                                        'operator': 'Equal',
                                        'value': 'value1',
                                    },
                                    {
                                        'key': 'key2',
                                        'operator': 'Equal',
                                        'value': 'value2',
                                    },
                                ]
                            }
                        }
                    }
                }
            }
        }

    def test_respects_other_configuration(self):

        @dsl.pipeline
        def my_pipeline():
            task = comp()
            kubernetes.use_secret_as_volume(
                task, secret_name='my-secret', mount_path='/mnt/my_vol')
            kubernetes.add_toleration(
                task,
                key='key1',
                operator='Equal',
                value='value1',
            )

        assert json_format.MessageToDict(my_pipeline.platform_spec) == {
            'platforms': {
                'kubernetes': {
                    'deploymentSpec': {
                        'executors': {
                            'exec-comp': {
                                'tolerations': [{
                                    'key': 'key1',
                                    'operator': 'Equal',
                                    'value': 'value1',
                                },],
                                'secretAsVolume': [{
                                    'secretName': 'my-secret',
                                    'mountPath': '/mnt/my_vol',
                                    'optional': False
                                },],
                            },
                        }
                    }
                }
            }
        }


@dsl.component
def comp():
    pass



# Copyright 2023 The Kubeflow Authors
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from typing import Dict, List, Optional, Union

from google.protobuf import json_format
from google.protobuf import message
from kfp import dsl
from kfp.dsl import PipelineTask
from kfp.kubernetes import common
from kfp.kubernetes import kubernetes_executor_config_pb2 as pb


@dsl.container_component
def CreatePVC(
    name: dsl.OutputPath(str),
    access_modes: List[str],
    size: str,
    pvc_name: Optional[str] = None,
    pvc_name_suffix: Optional[str] = None,
    storage_class_name: Optional[str] = '',
    volume_name: Optional[str] = None,
    annotations: Optional[Dict[str, str]] = None,
):
    """Create a PersistentVolumeClaim, which can be used by downstream tasks.
    See `PersistentVolume <https://kubernetes.io/docs/concepts/storage/persistent-volumes/#persistent-volumes>`_ and `PersistentVolumeClaim <https://kubernetes.io/docs/concepts/storage/persistent-volumes/#persistentvolumeclaims>`_ documentation for more information about
    the component input parameters.

    Args:
        access_modes: AccessModes to request for the provisioned PVC. May
            be one or more of ``'ReadWriteOnce'``, ``'ReadOnlyMany'``, ``'ReadWriteMany'``, or
            ``'ReadWriteOncePod'``. Corresponds to `PersistentVolumeClaim.spec.accessModes <https://kubernetes.io/docs/concepts/storage/persistent-volumes/#access-modes>`_.
        size: The size of storage requested by the PVC that will be provisioned. For example, ``'5Gi'``. Corresponds to `PersistentVolumeClaim.spec.resources.requests.storage <https://kubernetes.io/docs/reference/kubernetes-api/config-and-storage-resources/persistent-volume-claim-v1/#PersistentVolumeClaimSpec>`_.
        pvc_name: Name of the PVC. Corresponds to `PersistentVolumeClaim.metadata.name <https://kubernetes.io/docs/reference/kubernetes-api/config-and-storage-resources/persistent-volume-claim-v1/#PersistentVolumeClaim>`_. Only one of ``pvc_name`` and ``pvc_name_suffix`` can
            be provided.
        pvc_name_suffix: Prefix to use for a dynamically generated name, which
            will take the form ``<argo-workflow-name>-<pvc_name_suffix>``. Only one
            of ``pvc_name`` and ``pvc_name_suffix`` can be provided.
        storage_class_name: Name of StorageClass from which to provision the PV
            to back the PVC. ``None`` indicates to use the cluster's default
            storage_class_name. Set to ``''`` for a statically specified PVC.
        volume_name: Pre-existing PersistentVolume that should back the
            provisioned PersistentVolumeClaim. Used for statically
            specified PV only. Corresponds to `PersistentVolumeClaim.spec.volumeName <https://kubernetes.io/docs/reference/kubernetes-api/config-and-storage-resources/persistent-volume-claim-v1/#PersistentVolumeClaimSpec>`_.
        annotations: Annotations for the PVC's metadata. Corresponds to `PersistentVolumeClaim.metadata.annotations <https://kubernetes.io/docs/reference/kubernetes-api/config-and-storage-resources/persistent-volume-claim-v1/#PersistentVolumeClaim>`_.

    Returns:
        ``name: str`` \n\t\t\tName of the generated PVC.
    """

    return dsl.ContainerSpec(image='argostub/createpvc')


def mount_pvc(
    task: PipelineTask,
    pvc_name: Union[str, 'PipelineChannel'],
    mount_path: str,
) -> PipelineTask:
    """Mount a PersistentVolumeClaim to the task's container.

    Args:
        task: Pipeline task.
        pvc_name: Name of the PVC to mount. Supports passing a runtime-generated name, such as a name provided by ``kubernetes.CreatePvcOp().outputs['name']``.
        mount_path: Path to which the PVC should be mounted as a volume.

    Returns:
        Task object with updated PVC mount configuration.
    """

    msg = common.get_existing_kubernetes_config_as_message(task)

    pvc_mount = pb.PvcMount(mount_path=mount_path)
    pvc_name_from_task = _assign_pvc_name_to_msg(pvc_mount, pvc_name)
    if pvc_name_from_task:
        task.after(pvc_name.task)

    msg.pvc_mount.append(pvc_mount)
    task.platform_config['kubernetes'] = json_format.MessageToDict(msg)

    return task


@dsl.container_component
def DeletePVC(pvc_name: str):
    """Delete a PersistentVolumeClaim.

    Args:
        pvc_name: Name of the PVC to delete. Supports passing a runtime-generated name, such as a name provided by ``kubernetes.CreatePvcOp().outputs['name']``.
    """
    return dsl.ContainerSpec(image='argostub/deletepvc')


def _assign_pvc_name_to_msg(
    msg: message.Message,
    pvc_name: Union[str, 'PipelineChannel'],
) -> bool:
    """Assigns pvc_name to the msg's pvc_reference oneof. Returns True if pvc_name is an upstream task output. Else, returns False."""
    if isinstance(pvc_name, str):
        msg.constant = pvc_name
        return False
    elif hasattr(pvc_name, 'task_name'):
        if pvc_name.task_name is None:
            msg.component_input_parameter = pvc_name.name
            return False
        else:
            msg.task_output_parameter.producer_task = pvc_name.task_name
            msg.task_output_parameter.output_parameter_key = pvc_name.name
            return True
    else:
        raise ValueError(
            f'Argument for {"pvc_name"!r} must be an instance of str or PipelineChannel. Got unknown input type: {type(pvc_name)!r}. '
        )


def add_ephemeral_volume(
    task: PipelineTask,
    volume_name: str,
    mount_path: str,
    access_modes: List[str],
    size: str,
    storage_class_name: Optional[str] = None,
    labels: Dict[str, str] = None,
    annotations: Dict[str, str] = None,
):
    """Add a `generic ephemeral volume
    <https://kubernetes.io/docs/concepts/storage/ephemeral-volumes/#generic-ephemeral-volumes>`_. to a task.

    Args:
        task:
            Pipeline task.
        volume_name:
            name to be given to the created ephemeral volume. Corresponds to Pod.spec.volumes[*].name
        mount_path:
            local path in the main container where the PVC should be mounted as a volume
        access_modes:
            AccessModes to request for the provisioned PVC. May be one or more of ``'ReadWriteOnce'``,
            ``'ReadOnlyMany'``, ``'ReadWriteMany'``, or``'ReadWriteOncePod'``. Corresponds to
            `Pod.spec.volumes[*].ephemeral.volumeClaimTemplate.spec.accessModes
            <https://kubernetes.io/docs/concepts/storage/persistent-volumes/#access-modes>`_.
        size:
            The size of storage requested by the PVC that will be provisioned. For example, ``'5Gi'``. Corresponds to
            `Pod.spec.volumes[*].ephemeral.volumeClaimTemplate.spec.resources.requests.storage
            <https://kubernetes.io/docs/reference/kubernetes-api/config-and-storage-resources/persistent-volume-claim-v1/#PersistentVolumeClaimSpec>`_.
        storage_class_name:
            Name of StorageClass from which to provision the PV to back the PVC. ``None`` indicates to use the
            cluster's default storage_class_name.
        labels:
            The labels to attach to the created PVC. Corresponds to
            `Pod.spec.volumes[*].ephemeral.volumeClaimTemplate.metadata.labels
        annotations:
            The annotation to attach to the created PVC. Corresponds to
            `Pod.spec.volumes[*].ephemeral.volumeClaimTemplate.metadata.annotations
    Returns:
        Task object with added toleration.
    """

    msg = common.get_existing_kubernetes_config_as_message(task)
    msg.generic_ephemeral_volume.append(
        pb.GenericEphemeralVolume(
            volume_name=volume_name,
            mount_path=mount_path,
            access_modes=access_modes,
            size=size,
            default_storage_class=storage_class_name is None,
            storage_class_name=storage_class_name,
            metadata=pb.PodMetadata(
                annotations=annotations or {},
                labels=labels or {},
            ) if annotations or labels else None,
        )
    )
    task.platform_config["kubernetes"] = json_format.MessageToDict(msg)

    return task


# Copyright 2024 The Kubeflow Authors
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from google.protobuf import json_format
from kfp import dsl
from kfp import kubernetes
import pytest


class TestTimeout:

    def test_timeout(self):

        @dsl.pipeline
        def my_pipeline():
            task = comp()
            kubernetes.set_timeout(
                task,
                seconds=20
            )

        assert json_format.MessageToDict(my_pipeline.platform_spec) == {
            'platforms': {
                'kubernetes': {
                    'deploymentSpec': {
                        'executors': {
                            'exec-comp': {
                                'activeDeadlineSeconds': '20'
                            }
                        }
                    }
                }
            }
        }

    def test_reset_timeout(self):

        @dsl.pipeline
        def my_pipeline():
            task = comp()
            kubernetes.set_timeout(
                task,
                seconds=20
            )
            kubernetes.set_timeout(
                task,
                seconds=0
            )

        assert json_format.MessageToDict(my_pipeline.platform_spec) == {
            'platforms': {
                'kubernetes': {
                    'deploymentSpec': {
                        'executors': {
                            'exec-comp': {
                            }
                        }
                    }
                }
            }
        }

    def test_bad_value_timeout(self):

        with pytest.raises(
                ValueError,
                match=r'Argument for "seconds" must be an integer greater or equals to 0. Got invalid input: -20.',
        ):
            
            @dsl.pipeline
            def my_pipeline():
                task = comp()
                kubernetes.set_timeout(
                    task,
                    seconds=-20
                )


@dsl.component
def comp():
    pass



# Copyright 2024 The Kubeflow Authors
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from kfp import dsl
from kfp import kubernetes


@dsl.component
def comp():
    pass


@dsl.pipeline
def my_pipeline():
    task = comp()
    kubernetes.use_config_map_as_env(
        task,
        config_map_name='my-cm',
        config_map_key_to_env={'foo': 'CONFIG_MAP_VAR'})


if __name__ == '__main__':
    from kfp import compiler
    compiler.Compiler().compile(my_pipeline, __file__.replace('.py', '.yaml'))



# Copyright 2023 The Kubeflow Authors
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from google.protobuf import json_format
from kfp.dsl import PipelineTask
from kfp.kubernetes import common


def add_node_selector(
    task: PipelineTask,
    label_key: str,
    label_value: str,
) -> PipelineTask:
    """Add a constraint to the task Pod's `nodeSelector
    <https://kubernetes.io/docs/concepts/scheduling-eviction/assign-pod-node/#nodeselector>`_.

    Each constraint is a key-value pair, corresponding to the PodSpec's `nodeSelector <https://kubernetes.io/docs/reference/kubernetes-api/workload-resources/pod-v1/#scheduling>`_ field.

    For the task's Pod to be eligible to run on a node, the node's labels must satisfy the constraint.

    Args:
        task: Pipeline task.
        label_key: Key of the nodeSelector label.
        label_value: Value of the nodeSelector label.

    Returns:
        Task object with an added nodeSelector constraint.
    """

    msg = common.get_existing_kubernetes_config_as_message(task)
    msg.node_selector.labels.update({label_key: label_value})
    task.platform_config['kubernetes'] = json_format.MessageToDict(msg)

    return task



# Copyright 2024 The Kubeflow Authors
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from typing import Optional

from google.protobuf import json_format
from kfp.dsl import PipelineTask
from kfp.kubernetes import common
from kfp.kubernetes import kubernetes_executor_config_pb2 as pb

try:
    from typing import Literal
except ImportError:
    from typing_extensions import Literal


def add_toleration(
    task: PipelineTask,
    key: Optional[str] = None,
    operator: Optional[Literal["Equal", "Exists"]] = None,
    value: Optional[str] = None,
    effect: Optional[Literal["NoExecute", "NoSchedule", "PreferNoSchedule"]] = None,
    toleration_seconds: Optional[int] = None,
):
    """Add a `toleration<https://kubernetes.io/docs/concepts/scheduling-eviction/taint-and-toleration/>`_. to a task.

    Args:
        task:
            Pipeline task.
        key:
            key is the taint key that the toleration applies to. Empty means
            match all taint keys. If the key is empty, operator must be Exists;
            this combination means to match all values and all keys.
        operator:
            operator represents a key's relationship to the value. Valid
            operators are Exists and Equal. Defaults to Equal. Exists is
            equivalent to wildcard for value, so that a pod can tolerate all
            taints of a particular category.
        value:
            value is the taint value the toleration matches to. If the operator
            is Exists, the value should be empty, otherwise just a regular
            string.
        effect:
            effect indicates the taint effect to match. Empty means match all
            taint effects. When specified, allowed values are NoSchedule,
            PreferNoSchedule and NoExecute.
        toleration_seconds:
            toleration_seconds represents the period of time the toleration
            (which must be of effect NoExecute, otherwise this field is ignored)
            tolerates the taint. By default, it is not set, which means tolerate
            the taint forever (do not evict). Zero and negative values will be
            treated as 0 (evict immediately) by the system.

    Returns:
        Task object with added toleration.
    """

    msg = common.get_existing_kubernetes_config_as_message(task)
    msg.tolerations.append(
        pb.Toleration(
            key=key,
            operator=operator,
            value=value,
            effect=effect,
            toleration_seconds=toleration_seconds,
        )
    )
    task.platform_config["kubernetes"] = json_format.MessageToDict(msg)

    return task



# Copyright 2023 The Kubeflow Authors
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from google.protobuf import json_format
from kfp import dsl
from kfp import kubernetes


class TestUseSecretAsVolume:

    def test_use_one(self):

        @dsl.pipeline
        def my_pipeline():
            task = comp()
            kubernetes.use_secret_as_volume(
                task,
                secret_name='secret-name',
                mount_path='secretpath',
            )

        assert json_format.MessageToDict(my_pipeline.platform_spec) == {
            'platforms': {
                'kubernetes': {
                    'deploymentSpec': {
                        'executors': {
                            'exec-comp': {
                                'secretAsVolume': [{
                                    'secretName': 'secret-name',
                                    'mountPath': 'secretpath',
                                    'optional': False
                                }]
                            }
                        }
                    }
                }
            }
        }

    def test_use_one_optional_true(self):

        @dsl.pipeline
        def my_pipeline():
            task = comp()
            kubernetes.use_secret_as_volume(
                task,
                secret_name='secret-name',
                mount_path='secretpath',
                optional=True)

        assert json_format.MessageToDict(my_pipeline.platform_spec) == {
            'platforms': {
                'kubernetes': {
                    'deploymentSpec': {
                        'executors': {
                            'exec-comp': {
                                'secretAsVolume': [{
                                    'secretName': 'secret-name',
                                    'mountPath': 'secretpath',
                                    'optional': True
                                }]
                            }
                        }
                    }
                }
            }
        }

    def test_use_one_optional_false(self):

        @dsl.pipeline
        def my_pipeline():
            task = comp()
            kubernetes.use_secret_as_volume(
                task,
                secret_name='secret-name',
                mount_path='secretpath',
                optional=False)

        assert json_format.MessageToDict(my_pipeline.platform_spec) == {
            'platforms': {
                'kubernetes': {
                    'deploymentSpec': {
                        'executors': {
                            'exec-comp': {
                                'secretAsVolume': [{
                                    'secretName': 'secret-name',
                                    'mountPath': 'secretpath',
                                    'optional': False
                                }]
                            }
                        }
                    }
                }
            }
        }

    def test_use_two(self):

        @dsl.pipeline
        def my_pipeline():
            task = comp()
            kubernetes.use_secret_as_volume(
                task,
                secret_name='secret-name1',
                mount_path='secretpath1',
            )
            kubernetes.use_secret_as_volume(
                task,
                secret_name='secret-name2',
                mount_path='secretpath2',
            )

        assert json_format.MessageToDict(my_pipeline.platform_spec) == {
            'platforms': {
                'kubernetes': {
                    'deploymentSpec': {
                        'executors': {
                            'exec-comp': {
                                'secretAsVolume': [
                                    {
                                        'secretName': 'secret-name1',
                                        'mountPath': 'secretpath1',
                                        'optional': False
                                    },
                                    {
                                        'secretName': 'secret-name2',
                                        'mountPath': 'secretpath2',
                                        'optional': False
                                    },
                                ]
                            }
                        }
                    }
                }
            }
        }

    def test_preserves_secret_as_env(self):
        # checks that use_secret_as_volume respects previously set secrets as env

        @dsl.pipeline
        def my_pipeline():
            task = comp()
            kubernetes.use_secret_as_env(
                task,
                secret_name='secret-name1',
                secret_key_to_env={'password': 'SECRET_VAR'},
            )
            kubernetes.use_secret_as_volume(
                task,
                secret_name='secret-name2',
                mount_path='secretpath2',
            )

        assert json_format.MessageToDict(my_pipeline.platform_spec) == {
            'platforms': {
                'kubernetes': {
                    'deploymentSpec': {
                        'executors': {
                            'exec-comp': {
                                'secretAsEnv': [{
                                    'secretName':
                                        'secret-name1',
                                    'keyToEnv': [{
                                        'secretKey': 'password',
                                        'envVar': 'SECRET_VAR'
                                    }]
                                }],
                                'secretAsVolume': [{
                                    'secretName': 'secret-name2',
                                    'mountPath': 'secretpath2',
                                    'optional': False
                                },]
                            }
                        }
                    }
                }
            }
        }

    def test_alongside_pvc_mount(self):
        # checks that use_secret_as_volume respects previously set pvc
        @dsl.pipeline
        def my_pipeline():
            task = comp()
            kubernetes.mount_pvc(
                task,
                pvc_name='pvc-name',
                mount_path='path',
            )
            kubernetes.use_secret_as_volume(
                task,
                secret_name='secret-name',
                mount_path='secretpath',
            )

        assert json_format.MessageToDict(my_pipeline.platform_spec) == {
            'platforms': {
                'kubernetes': {
                    'deploymentSpec': {
                        'executors': {
                            'exec-comp': {
                                'pvcMount': [{
                                    'constant': 'pvc-name',
                                    'mountPath': 'path'
                                }],
                                'secretAsVolume': [{
                                    'secretName': 'secret-name',
                                    'mountPath': 'secretpath',
                                    'optional': False
                                }]
                            }
                        }
                    }
                }
            }
        }


class TestUseSecretAsEnv:

    def test_use_one(self):

        @dsl.pipeline
        def my_pipeline():
            task = comp()
            kubernetes.use_secret_as_env(
                task,
                secret_name='secret-name',
                secret_key_to_env={
                    'username': 'USERNAME',
                    'password': 'PASSWORD',
                },
            )

        assert json_format.MessageToDict(my_pipeline.platform_spec) == {
            'platforms': {
                'kubernetes': {
                    'deploymentSpec': {
                        'executors': {
                            'exec-comp': {
                                'secretAsEnv': [{
                                    'secretName':
                                        'secret-name',
                                    'keyToEnv': [
                                        {
                                            'secretKey': 'username',
                                            'envVar': 'USERNAME'
                                        },
                                        {
                                            'secretKey': 'password',
                                            'envVar': 'PASSWORD'
                                        },
                                    ]
                                }]
                            }
                        }
                    }
                }
            }
        }

    def test_use_two(self):

        @dsl.pipeline
        def my_pipeline():
            task = comp()
            kubernetes.use_secret_as_env(
                task,
                secret_name='secret-name1',
                secret_key_to_env={'password1': 'SECRET_VAR1'},
            )
            kubernetes.use_secret_as_env(
                task,
                secret_name='secret-name2',
                secret_key_to_env={'password2': 'SECRET_VAR2'},
            )

        assert json_format.MessageToDict(my_pipeline.platform_spec) == {
            'platforms': {
                'kubernetes': {
                    'deploymentSpec': {
                        'executors': {
                            'exec-comp': {
                                'secretAsEnv': [
                                    {
                                        'secretName':
                                            'secret-name1',
                                        'keyToEnv': [{
                                            'secretKey': 'password1',
                                            'envVar': 'SECRET_VAR1'
                                        }]
                                    },
                                    {
                                        'secretName':
                                            'secret-name2',
                                        'keyToEnv': [{
                                            'secretKey': 'password2',
                                            'envVar': 'SECRET_VAR2'
                                        }]
                                    },
                                ]
                            }
                        }
                    }
                }
            }
        }

    def test_preserves_secret_as_volume(self):
        # checks that use_secret_as_env respects previously set secrets as vol

        @dsl.pipeline
        def my_pipeline():
            task = comp()
            kubernetes.use_secret_as_volume(
                task,
                secret_name='secret-name2',
                mount_path='secretpath2',
            )
            kubernetes.use_secret_as_env(
                task,
                secret_name='secret-name1',
                secret_key_to_env={'password': 'SECRET_VAR'},
            )

        assert json_format.MessageToDict(my_pipeline.platform_spec) == {
            'platforms': {
                'kubernetes': {
                    'deploymentSpec': {
                        'executors': {
                            'exec-comp': {
                                'secretAsEnv': [{
                                    'secretName':
                                        'secret-name1',
                                    'keyToEnv': [{
                                        'secretKey': 'password',
                                        'envVar': 'SECRET_VAR'
                                    }]
                                }],
                                'secretAsVolume': [{
                                    'secretName': 'secret-name2',
                                    'mountPath': 'secretpath2',
                                    'optional': False
                                },]
                            }
                        }
                    }
                }
            }
        }

    def test_preserves_pvc_mount(self):
        # checks that use_secret_as_env respects previously set pvc
        @dsl.pipeline
        def my_pipeline():
            task = comp()
            kubernetes.mount_pvc(
                task,
                pvc_name='pvc-name',
                mount_path='path',
            )
            kubernetes.use_secret_as_env(
                task,
                secret_name='secret-name',
                secret_key_to_env={'password': 'SECRET_VAR'},
            )

        assert json_format.MessageToDict(my_pipeline.platform_spec) == {
            'platforms': {
                'kubernetes': {
                    'deploymentSpec': {
                        'executors': {
                            'exec-comp': {
                                'pvcMount': [{
                                    'constant': 'pvc-name',
                                    'mountPath': 'path'
                                }],
                                'secretAsEnv': [{
                                    'secretName':
                                        'secret-name',
                                    'keyToEnv': [{
                                        'secretKey': 'password',
                                        'envVar': 'SECRET_VAR'
                                    }]
                                }]
                            }
                        }
                    }
                }
            }
        }


@dsl.component
def comp():
    pass



# Copyright 2023 The Kubeflow Authors
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from google.protobuf import json_format
from kfp.kubernetes import kubernetes_executor_config_pb2 as pb


def get_existing_kubernetes_config_as_message(
        task: 'PipelineTask') -> pb.KubernetesExecutorConfig:
    cur_k8_config_dict = task.platform_config.get('kubernetes', {})
    k8_config_msg = pb.KubernetesExecutorConfig()
    return json_format.ParseDict(cur_k8_config_dict, k8_config_msg)



# Copyright 2023 The Kubeflow Authors
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import os
import re
import sys
import tempfile
from typing import Any, Callable, Dict, List, Tuple, Union
import unittest

from kfp import compiler
from kfp import components
from kfp.components import placeholders
import pytest
import yaml

_TEST_ROOT_DIR = os.path.dirname(__file__)
_CONFIG_PATH = os.path.join(_TEST_ROOT_DIR, 'test_data_config.yaml')
_TEST_DATA_DIR = os.path.join(_TEST_ROOT_DIR, 'data')


def create_test_cases() -> List[Dict[str, Any]]:
    with open(_CONFIG_PATH) as f:
        config = yaml.safe_load(f)
    return [(test_case['module'], test_case['name'])
            for test_case in config['test_cases']]


def import_obj_from_file(python_path: str, obj_name: str) -> Any:
    sys.path.insert(0, os.path.dirname(python_path))
    module_name = os.path.splitext(os.path.split(python_path)[1])[0]
    module = __import__(module_name, fromlist=[obj_name])
    if not hasattr(module, obj_name):
        raise ValueError(
            f'Object "{obj_name}" not found in module {python_path}.')
    return getattr(module, obj_name)


def ignore_kfp_version_helper(spec: Dict[str, Any]) -> Dict[str, Any]:
    """Ignores kfp sdk versioning in command.

    Takes in a YAML input and ignores the kfp sdk versioning in command
    for comparison between compiled file and goldens.
    """
    pipeline_spec = spec.get('pipelineSpec', spec)

    if 'executors' in pipeline_spec['deploymentSpec']:
        for executor in pipeline_spec['deploymentSpec']['executors']:
            pipeline_spec['deploymentSpec']['executors'][
                executor] = yaml.safe_load(
                    re.sub(
                        r"'kfp==(\d+).(\d+).(\d+)(-[a-z]+.\d+)?'", 'kfp',
                        yaml.dump(
                            pipeline_spec['deploymentSpec']['executors']
                            [executor],
                            sort_keys=True)))
    return spec


def load_pipeline_spec_and_platform_spec(
        filename: str) -> Tuple[Dict[str, Any], Dict[str, Any]]:
    with open(filename) as f:
        pipeline_spec, platform_spec = tuple(yaml.safe_load_all(f))
        # ignore the sdkVersion
        del pipeline_spec['sdkVersion']
        return ignore_kfp_version_helper(pipeline_spec), platform_spec


def handle_placeholders(
        component_spec: 'structures.ComponentSpec'
) -> 'structures.ComponentSpec':
    if component_spec.implementation.container is not None:
        if component_spec.implementation.container.command is not None:
            component_spec.implementation.container.command = [
                placeholders.convert_command_line_element_to_string(c)
                for c in component_spec.implementation.container.command
            ]
        if component_spec.implementation.container.args is not None:
            component_spec.implementation.container.args = [
                placeholders.convert_command_line_element_to_string(a)
                for a in component_spec.implementation.container.args
            ]
    return component_spec


def handle_expected_diffs(
        component_spec: 'structures.ComponentSpec'
) -> 'structures.ComponentSpec':
    """Strips some component spec fields that should be ignored when comparing
    with golden result."""
    # Ignore description when comparing components specs read in from v1 component YAML and from IR YAML, because non lightweight Python components defined in v1 YAML can have a description field, but IR YAML does not preserve this field unless the component is a lightweight Python function-based component
    component_spec.description = None
    # ignore SDK version so that golden snapshots don't need to be updated between SDK version bump
    if component_spec.implementation.graph is not None:
        component_spec.implementation.graph.sdk_version = ''

    return handle_placeholders(component_spec)


class TestReadWrite:

    def _compile_and_load_component(
        self, compilable: Union[Callable[..., Any],
                                'python_component.PythonComponent']):
        with tempfile.TemporaryDirectory() as tmp_dir:
            tmp_file = os.path.join(tmp_dir, 're_compiled_output.yaml')
            compiler.Compiler().compile(compilable, tmp_file)
            return components.load_component_from_file(tmp_file)

    def _compile_and_read_yaml(
        self, compilable: Union[Callable[..., Any],
                                'python_component.PythonComponent']
    ) -> Tuple[Dict[str, Any], Dict[str, Any]]:
        with tempfile.TemporaryDirectory() as tmp_dir:
            tmp_file = os.path.join(tmp_dir, 're_compiled_output.yaml')
            compiler.Compiler().compile(compilable, tmp_file)
            return load_pipeline_spec_and_platform_spec(tmp_file)

    def _test_serialization_deserialization_consistency(self, yaml_file: str):
        """Tests serialization and deserialization consistency."""
        original_component = components.load_component_from_file(yaml_file)
        reloaded_component = self._compile_and_load_component(
            original_component)
        assert handle_expected_diffs(
            original_component.component_spec) == handle_expected_diffs(
                reloaded_component.component_spec)

    def _test_serialization_correctness(
        self,
        python_file: str,
        yaml_file: str,
        function_name: str,
    ):
        """Tests serialization correctness."""
        pipeline = import_obj_from_file(python_file, function_name)
        compiled_pipeline_spec, compiled_platform_spec = self._compile_and_read_yaml(
            pipeline)
        golden_pipeline_spec, golden_platform_spec = load_pipeline_spec_and_platform_spec(
            yaml_file)
        assert compiled_pipeline_spec == golden_pipeline_spec
        assert compiled_platform_spec == golden_platform_spec

    @pytest.mark.parametrize('test_case,function', create_test_cases())
    def test(
        self,
        test_case: str,
        function: str,
    ):
        """Tests serialization and deserialization consistency and correctness.

        Args:
            name (str): '{test_group_name}-{test_case_name}'. Useful for print statements/debugging.
            test_case (str): Test case name (without file extension).
            test_data_dir (str): The directory containing the test case files.
            function (str, optional): The function name to compile.
            read (bool): Whether the pipeline/component supports deserialization from YAML (IR, except for V1 component YAML back compatability tests).
            write (bool): Whether the pipeline/component supports compilation from a Python file.
        """
        yaml_file = os.path.join(_TEST_DATA_DIR, f'{test_case}.yaml')
        py_file = os.path.join(_TEST_DATA_DIR, f'{test_case}.py')

        self._test_serialization_correctness(
            python_file=py_file, yaml_file=yaml_file, function_name=function)

        self._test_serialization_deserialization_consistency(
            yaml_file=yaml_file)


if __name__ == '__main__':
    unittest.main()



# Copyright 2024 The Kubeflow Authors
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from typing import List

from google.protobuf import json_format
from kfp.dsl import PipelineTask
from kfp.kubernetes import common
from kfp.kubernetes import kubernetes_executor_config_pb2 as pb


def set_image_pull_secrets(
    task: PipelineTask,
    secret_names: List[str],
) -> PipelineTask:
    """Set image pull secrets for a Kubernetes task.

    Args:
        task: Pipeline task.
        secret_names: List of image pull secret names.

    Returns:
        Task object with updated image pull secret configuration.
    """

    msg = common.get_existing_kubernetes_config_as_message(task)

    # Assuming secret_names is a list of strings
    image_pull_secret = [
        pb.ImagePullSecret(secret_name=secret_name)
        for secret_name in secret_names
    ]

    msg.image_pull_secret.extend(image_pull_secret)

    task.platform_config['kubernetes'] = json_format.MessageToDict(msg)

    return task


def set_image_pull_policy(task: PipelineTask, policy: str) -> PipelineTask:
    """Set image pull policy for the container.

    Args:
        task: Pipeline task.
        policy: One of `Always`, `Never`, `IfNotPresent`.

    Returns:
        Task object with an added ImagePullPolicy specification.
    """
    if policy not in ['Always', 'Never', 'IfNotPresent']:
        raise ValueError(
            'Invalid imagePullPolicy. Must be one of `Always`, `Never`, `IfNotPresent`.'
        )
    msg = common.get_existing_kubernetes_config_as_message(task)
    msg.image_pull_policy = policy
    task.platform_config['kubernetes'] = json_format.MessageToDict(msg)

    return task



# Copyright 2023 The Kubeflow Authors
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import os
import re

import setuptools

NAME = 'kfp-kubernetes'
REQUIREMENTS = [
    'protobuf>=4.21.1,<5',
    'kfp>=2.6.0,<3',
]
DEV_REQUIREMENTS = [
    'docformatter==1.4',
    'isort==5.10.1',
    'mypy==0.941',
    'pre-commit==2.19.0',
    'pycln==2.1.1',
    'pytest==7.1.2',
    'pytest-xdist==2.5.0',
    'yapf==0.32.0',
]


def find_version(*file_path_parts: str) -> str:
    """Get version from kfp.__init__.__version__."""

    file_path = os.path.join(os.path.dirname(__file__), *file_path_parts)
    with open(file_path, 'r') as f:
        version_file_text = f.read()

    version_match = re.search(
        r"^__version__ = ['\"]([^'\"]*)['\"]",
        version_file_text,
        re.M,
    )
    if version_match:
        return version_match[1]
    else:
        raise ValueError('Could not find version.')


def read_readme() -> str:
    readme_path = os.path.join(os.path.dirname(__file__), 'README.md')
    with open(readme_path) as f:
        return f.read()


setuptools.setup(
    name=NAME,
    version=find_version('kfp', 'kubernetes', '__init__.py'),
    description='Kubernetes platform configuration library and generated protos.',
    long_description=read_readme(),
    long_description_content_type='text/markdown',
    author='google',
    author_email='kubeflow-pipelines@google.com',
    url='https://github.com/kubeflow/pipelines',
    project_urls={
        'Documentation':
            'https://kfp-kubernetes.readthedocs.io/',
        'Bug Tracker':
            'https://github.com/kubeflow/pipelines/issues',
        'Source':
            'https://github.com/kubeflow/pipelines/tree/master/kubernetes_platform/python',
    },
    packages=setuptools.find_namespace_packages(include=['kfp.*']),
    python_requires='>=3.9.0',
    install_requires=REQUIREMENTS,
    include_package_data=True,
    extras_require={
        'dev': DEV_REQUIREMENTS,
    },
    license='Apache 2.0',
)



# Copyright 2024 The Kubeflow Authors
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from google.protobuf import json_format
from kfp.dsl import PipelineTask
from kfp.kubernetes import common
from kfp.kubernetes import kubernetes_executor_config_pb2 as pb


def use_field_path_as_env(
    task: PipelineTask,
    env_name: str,
    field_path: str,
) -> PipelineTask:
    """Use a Kubernetes Field Path as an environment variable as described in
    https://kubernetes.io/docs/tasks/inject-data-application/environment-variable-expose-pod-information

    Args:
        task: Pipeline task.
        env_name: Name of the enviornment variable.
        field_path: Kubernetes field path to expose as the enviornment variable.

    Returns:
        Task object with updated field path as the enviornment variable.
    """

    msg = common.get_existing_kubernetes_config_as_message(task)
    field_path_as_env = pb.FieldPathAsEnv(
        name=env_name,
        field_path=field_path,
    )
    msg.field_path_as_env.append(field_path_as_env)
    task.platform_config['kubernetes'] = json_format.MessageToDict(msg)

    return task



# Copyright 2024 The Kubeflow Authors
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from typing import Optional

from google.protobuf import json_format
from kfp.dsl import PipelineTask
from kfp.kubernetes import common
from kfp.kubernetes import kubernetes_executor_config_pb2 as pb


def empty_dir_mount(
    task: PipelineTask,
    volume_name: str,
    mount_path: str,
    medium: Optional[str] = None,
    size_limit: Optional[str] = None,
) -> PipelineTask:
    """Mount an EmptyDir volume to the task's container.

    Args:
        task: Pipeline task.
        volume_name: Name of the EmptyDir volume. 
        mount_path: Path within the container at which the EmptyDir should be mounted.
        medium: Storage medium to back the EmptyDir. Must be one of `Memory` or `HugePages`. Defaults to `None`.
        size_limit: Maximum size of the EmptyDir. For example, `5Gi`. Defaults to `None`.

    Returns:
        Task object with updated EmptyDir mount configuration.
    """

    msg = common.get_existing_kubernetes_config_as_message(task)

    empty_dir_mount = pb.EmptyDirMount(
        volume_name=volume_name,
        mount_path=mount_path,
        medium=medium,
        size_limit=size_limit,
    )

    msg.empty_dir_mounts.append(empty_dir_mount)

    task.platform_config['kubernetes'] = json_format.MessageToDict(msg)

    return task



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: kubernetes_executor_config.proto
"""Generated protocol buffer code."""
from google.protobuf.internal import builder as _builder
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from google.protobuf import struct_pb2 as google_dot_protobuf_dot_struct__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n kubernetes_executor_config.proto\x12\x0ekfp_kubernetes\x1a\x1cgoogle/protobuf/struct.proto\"\xb4\x07\n\x18KubernetesExecutorConfig\x12\x38\n\x10secret_as_volume\x18\x01 \x03(\x0b\x32\x1e.kfp_kubernetes.SecretAsVolume\x12\x32\n\rsecret_as_env\x18\x02 \x03(\x0b\x32\x1b.kfp_kubernetes.SecretAsEnv\x12+\n\tpvc_mount\x18\x03 \x03(\x0b\x32\x18.kfp_kubernetes.PvcMount\x12\x33\n\rnode_selector\x18\x04 \x01(\x0b\x32\x1c.kfp_kubernetes.NodeSelector\x12\x31\n\x0cpod_metadata\x18\x05 \x01(\x0b\x32\x1b.kfp_kubernetes.PodMetadata\x12:\n\x11image_pull_secret\x18\x06 \x03(\x0b\x32\x1f.kfp_kubernetes.ImagePullSecret\x12\x19\n\x11image_pull_policy\x18\x07 \x01(\t\x12?\n\x14\x63onfig_map_as_volume\x18\x08 \x03(\x0b\x32!.kfp_kubernetes.ConfigMapAsVolume\x12\x39\n\x11\x63onfig_map_as_env\x18\t \x03(\x0b\x32\x1e.kfp_kubernetes.ConfigMapAsEnv\x12\x1f\n\x17\x61\x63tive_deadline_seconds\x18\n \x01(\x03\x12\x39\n\x11\x66ield_path_as_env\x18\x0b \x03(\x0b\x32\x1e.kfp_kubernetes.FieldPathAsEnv\x12/\n\x0btolerations\x18\x0c \x03(\x0b\x32\x1a.kfp_kubernetes.Toleration\x12H\n\x18generic_ephemeral_volume\x18\r \x03(\x0b\x32&.kfp_kubernetes.GenericEphemeralVolume\x12\x37\n\rnode_affinity\x18\x0e \x03(\x0b\x32 .kfp_kubernetes.NodeAffinityTerm\x12\x35\n\x0cpod_affinity\x18\x0f \x03(\x0b\x32\x1f.kfp_kubernetes.PodAffinityTerm\x12\x42\n\x15\x65nabled_shared_memory\x18\x10 \x01(\x0b\x32#.kfp_kubernetes.EnabledSharedMemory\x12\x37\n\x10\x65mpty_dir_mounts\x18\x11 \x03(\x0b\x32\x1d.kfp_kubernetes.EmptyDirMount\"8\n\x13\x45nabledSharedMemory\x12\x13\n\x0bvolume_name\x18\x01 \x01(\t\x12\x0c\n\x04size\x18\x02 \x01(\t\"]\n\x0eSecretAsVolume\x12\x13\n\x0bsecret_name\x18\x01 \x01(\t\x12\x12\n\nmount_path\x18\x02 \x01(\t\x12\x15\n\x08optional\x18\x03 \x01(\x08H\x00\x88\x01\x01\x42\x0b\n\t_optional\"\x9f\x01\n\x0bSecretAsEnv\x12\x13\n\x0bsecret_name\x18\x01 \x01(\t\x12\x41\n\nkey_to_env\x18\x02 \x03(\x0b\x32-.kfp_kubernetes.SecretAsEnv.SecretKeyToEnvMap\x1a\x38\n\x11SecretKeyToEnvMap\x12\x12\n\nsecret_key\x18\x01 \x01(\t\x12\x0f\n\x07\x65nv_var\x18\x02 \x01(\t\"N\n\x17TaskOutputParameterSpec\x12\x15\n\rproducer_task\x18\x01 \x01(\t\x12\x1c\n\x14output_parameter_key\x18\x02 \x01(\t\"\xb2\x01\n\x08PvcMount\x12H\n\x15task_output_parameter\x18\x01 \x01(\x0b\x32\'.kfp_kubernetes.TaskOutputParameterSpecH\x00\x12\x12\n\x08\x63onstant\x18\x02 \x01(\tH\x00\x12#\n\x19\x63omponent_input_parameter\x18\x03 \x01(\tH\x00\x12\x12\n\nmount_path\x18\x04 \x01(\tB\x0f\n\rpvc_reference\"\xe4\x01\n\tCreatePvc\x12\x12\n\x08pvc_name\x18\x01 \x01(\tH\x00\x12\x19\n\x0fpvc_name_suffix\x18\x02 \x01(\tH\x00\x12\x14\n\x0c\x61\x63\x63\x65ss_modes\x18\x03 \x03(\t\x12\x0c\n\x04size\x18\x04 \x01(\t\x12\x1d\n\x15\x64\x65\x66\x61ult_storage_class\x18\x05 \x01(\x08\x12\x1a\n\x12storage_class_name\x18\x06 \x01(\t\x12\x13\n\x0bvolume_name\x18\x07 \x01(\t\x12,\n\x0b\x61nnotations\x18\x08 \x01(\x0b\x32\x17.google.protobuf.StructB\x06\n\x04name\"\x9f\x01\n\tDeletePvc\x12H\n\x15task_output_parameter\x18\x01 \x01(\x0b\x32\'.kfp_kubernetes.TaskOutputParameterSpecH\x00\x12\x12\n\x08\x63onstant\x18\x02 \x01(\tH\x00\x12#\n\x19\x63omponent_input_parameter\x18\x03 \x01(\tH\x00\x42\x0f\n\rpvc_reference\"w\n\x0cNodeSelector\x12\x38\n\x06labels\x18\x01 \x03(\x0b\x32(.kfp_kubernetes.NodeSelector.LabelsEntry\x1a-\n\x0bLabelsEntry\x12\x0b\n\x03key\x18\x01 \x01(\t\x12\r\n\x05value\x18\x02 \x01(\t:\x02\x38\x01\"\xec\x01\n\x0bPodMetadata\x12\x37\n\x06labels\x18\x01 \x03(\x0b\x32\'.kfp_kubernetes.PodMetadata.LabelsEntry\x12\x41\n\x0b\x61nnotations\x18\x02 \x03(\x0b\x32,.kfp_kubernetes.PodMetadata.AnnotationsEntry\x1a-\n\x0bLabelsEntry\x12\x0b\n\x03key\x18\x01 \x01(\t\x12\r\n\x05value\x18\x02 \x01(\t:\x02\x38\x01\x1a\x32\n\x10\x41nnotationsEntry\x12\x0b\n\x03key\x18\x01 \x01(\t\x12\r\n\x05value\x18\x02 \x01(\t:\x02\x38\x01\"d\n\x11\x43onfigMapAsVolume\x12\x17\n\x0f\x63onfig_map_name\x18\x01 \x01(\t\x12\x12\n\nmount_path\x18\x02 \x01(\t\x12\x15\n\x08optional\x18\x03 \x01(\x08H\x00\x88\x01\x01\x42\x0b\n\t_optional\"\xb3\x01\n\x0e\x43onfigMapAsEnv\x12\x17\n\x0f\x63onfig_map_name\x18\x01 \x01(\t\x12G\n\nkey_to_env\x18\x02 \x03(\x0b\x32\x33.kfp_kubernetes.ConfigMapAsEnv.ConfigMapKeyToEnvMap\x1a?\n\x14\x43onfigMapKeyToEnvMap\x12\x16\n\x0e\x63onfig_map_key\x18\x01 \x01(\t\x12\x0f\n\x07\x65nv_var\x18\x02 \x01(\t\"\xcf\x01\n\x16GenericEphemeralVolume\x12\x13\n\x0bvolume_name\x18\x01 \x01(\t\x12\x12\n\nmount_path\x18\x02 \x01(\t\x12\x14\n\x0c\x61\x63\x63\x65ss_modes\x18\x03 \x03(\t\x12\x0c\n\x04size\x18\x04 \x01(\t\x12\x1d\n\x15\x64\x65\x66\x61ult_storage_class\x18\x05 \x01(\x08\x12\x1a\n\x12storage_class_name\x18\x06 \x01(\t\x12-\n\x08metadata\x18\x07 \x01(\x0b\x32\x1b.kfp_kubernetes.PodMetadata\"&\n\x0fImagePullSecret\x12\x13\n\x0bsecret_name\x18\x01 \x01(\t\"2\n\x0e\x46ieldPathAsEnv\x12\x0c\n\x04name\x18\x01 \x01(\t\x12\x12\n\nfield_path\x18\x02 \x01(\t\"\x82\x01\n\nToleration\x12\x0b\n\x03key\x18\x01 \x01(\t\x12\x10\n\x08operator\x18\x02 \x01(\t\x12\r\n\x05value\x18\x03 \x01(\t\x12\x0e\n\x06\x65\x66\x66\x65\x63t\x18\x04 \x01(\t\x12\x1f\n\x12toleration_seconds\x18\x05 \x01(\x03H\x00\x88\x01\x01\x42\x15\n\x13_toleration_seconds\"D\n\x13SelectorRequirement\x12\x0b\n\x03key\x18\x01 \x01(\t\x12\x10\n\x08operator\x18\x02 \x01(\t\x12\x0e\n\x06values\x18\x03 \x03(\t\"\xad\x01\n\x10NodeAffinityTerm\x12>\n\x11match_expressions\x18\x01 \x03(\x0b\x32#.kfp_kubernetes.SelectorRequirement\x12\x39\n\x0cmatch_fields\x18\x02 \x03(\x0b\x32#.kfp_kubernetes.SelectorRequirement\x12\x13\n\x06weight\x18\x03 \x01(\x05H\x00\x88\x01\x01\x42\t\n\x07_weight\"\xa3\x04\n\x0fPodAffinityTerm\x12\x42\n\x15match_pod_expressions\x18\x01 \x03(\x0b\x32#.kfp_kubernetes.SelectorRequirement\x12M\n\x10match_pod_labels\x18\x02 \x03(\x0b\x32\x33.kfp_kubernetes.PodAffinityTerm.MatchPodLabelsEntry\x12\x14\n\x0ctopology_key\x18\x03 \x01(\t\x12\x12\n\nnamespaces\x18\x04 \x03(\t\x12H\n\x1bmatch_namespace_expressions\x18\x05 \x03(\x0b\x32#.kfp_kubernetes.SelectorRequirement\x12Y\n\x16match_namespace_labels\x18\x06 \x03(\x0b\x32\x39.kfp_kubernetes.PodAffinityTerm.MatchNamespaceLabelsEntry\x12\x13\n\x06weight\x18\x07 \x01(\x05H\x00\x88\x01\x01\x12\x11\n\x04\x61nti\x18\x08 \x01(\x08H\x01\x88\x01\x01\x1a\x35\n\x13MatchPodLabelsEntry\x12\x0b\n\x03key\x18\x01 \x01(\t\x12\r\n\x05value\x18\x02 \x01(\t:\x02\x38\x01\x1a;\n\x19MatchNamespaceLabelsEntry\x12\x0b\n\x03key\x18\x01 \x01(\t\x12\r\n\x05value\x18\x02 \x01(\t:\x02\x38\x01\x42\t\n\x07_weightB\x07\n\x05_anti\"\x80\x01\n\rEmptyDirMount\x12\x13\n\x0bvolume_name\x18\x01 \x01(\t\x12\x12\n\nmount_path\x18\x02 \x01(\t\x12\x13\n\x06medium\x18\x03 \x01(\tH\x00\x88\x01\x01\x12\x17\n\nsize_limit\x18\x04 \x01(\tH\x01\x88\x01\x01\x42\t\n\x07_mediumB\r\n\x0b_size_limitBIZGgithub.com/kubeflow/pipelines/kubernetes_platform/go/kubernetesplatformb\x06proto3')

_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, globals())
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'kubernetes_executor_config_pb2', globals())
if _descriptor._USE_C_DESCRIPTORS == False:

  DESCRIPTOR._options = None
  DESCRIPTOR._serialized_options = b'ZGgithub.com/kubeflow/pipelines/kubernetes_platform/go/kubernetesplatform'
  _NODESELECTOR_LABELSENTRY._options = None
  _NODESELECTOR_LABELSENTRY._serialized_options = b'8\001'
  _PODMETADATA_LABELSENTRY._options = None
  _PODMETADATA_LABELSENTRY._serialized_options = b'8\001'
  _PODMETADATA_ANNOTATIONSENTRY._options = None
  _PODMETADATA_ANNOTATIONSENTRY._serialized_options = b'8\001'
  _PODAFFINITYTERM_MATCHPODLABELSENTRY._options = None
  _PODAFFINITYTERM_MATCHPODLABELSENTRY._serialized_options = b'8\001'
  _PODAFFINITYTERM_MATCHNAMESPACELABELSENTRY._options = None
  _PODAFFINITYTERM_MATCHNAMESPACELABELSENTRY._serialized_options = b'8\001'
  _KUBERNETESEXECUTORCONFIG._serialized_start=83
  _KUBERNETESEXECUTORCONFIG._serialized_end=1031
  _ENABLEDSHAREDMEMORY._serialized_start=1033
  _ENABLEDSHAREDMEMORY._serialized_end=1089
  _SECRETASVOLUME._serialized_start=1091
  _SECRETASVOLUME._serialized_end=1184
  _SECRETASENV._serialized_start=1187
  _SECRETASENV._serialized_end=1346
  _SECRETASENV_SECRETKEYTOENVMAP._serialized_start=1290
  _SECRETASENV_SECRETKEYTOENVMAP._serialized_end=1346
  _TASKOUTPUTPARAMETERSPEC._serialized_start=1348
  _TASKOUTPUTPARAMETERSPEC._serialized_end=1426
  _PVCMOUNT._serialized_start=1429
  _PVCMOUNT._serialized_end=1607
  _CREATEPVC._serialized_start=1610
  _CREATEPVC._serialized_end=1838
  _DELETEPVC._serialized_start=1841
  _DELETEPVC._serialized_end=2000
  _NODESELECTOR._serialized_start=2002
  _NODESELECTOR._serialized_end=2121
  _NODESELECTOR_LABELSENTRY._serialized_start=2076
  _NODESELECTOR_LABELSENTRY._serialized_end=2121
  _PODMETADATA._serialized_start=2124
  _PODMETADATA._serialized_end=2360
  _PODMETADATA_LABELSENTRY._serialized_start=2076
  _PODMETADATA_LABELSENTRY._serialized_end=2121
  _PODMETADATA_ANNOTATIONSENTRY._serialized_start=2310
  _PODMETADATA_ANNOTATIONSENTRY._serialized_end=2360
  _CONFIGMAPASVOLUME._serialized_start=2362
  _CONFIGMAPASVOLUME._serialized_end=2462
  _CONFIGMAPASENV._serialized_start=2465
  _CONFIGMAPASENV._serialized_end=2644
  _CONFIGMAPASENV_CONFIGMAPKEYTOENVMAP._serialized_start=2581
  _CONFIGMAPASENV_CONFIGMAPKEYTOENVMAP._serialized_end=2644
  _GENERICEPHEMERALVOLUME._serialized_start=2647
  _GENERICEPHEMERALVOLUME._serialized_end=2854
  _IMAGEPULLSECRET._serialized_start=2856
  _IMAGEPULLSECRET._serialized_end=2894
  _FIELDPATHASENV._serialized_start=2896
  _FIELDPATHASENV._serialized_end=2946
  _TOLERATION._serialized_start=2949
  _TOLERATION._serialized_end=3079
  _SELECTORREQUIREMENT._serialized_start=3081
  _SELECTORREQUIREMENT._serialized_end=3149
  _NODEAFFINITYTERM._serialized_start=3152
  _NODEAFFINITYTERM._serialized_end=3325
  _PODAFFINITYTERM._serialized_start=3328
  _PODAFFINITYTERM._serialized_end=3875
  _PODAFFINITYTERM_MATCHPODLABELSENTRY._serialized_start=3741
  _PODAFFINITYTERM_MATCHPODLABELSENTRY._serialized_end=3794
  _PODAFFINITYTERM_MATCHNAMESPACELABELSENTRY._serialized_start=3796
  _PODAFFINITYTERM_MATCHNAMESPACELABELSENTRY._serialized_end=3855
  _EMPTYDIRMOUNT._serialized_start=3878
  _EMPTYDIRMOUNT._serialized_end=4006
# @@protoc_insertion_point(module_scope)



# Copyright 2024 The Kubeflow Authors
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from google.protobuf import json_format
from kfp import dsl
from kfp import kubernetes


class TestPodMetadata:

    def test_add_one(self):

        @dsl.pipeline
        def my_pipeline():
            task = comp()
            kubernetes.add_pod_label(
                task,
                label_key='kubeflow.com/kfp',
                label_value='pipeline-node',
            )

        assert json_format.MessageToDict(my_pipeline.platform_spec) == {
            'platforms': {
                'kubernetes': {
                    'deploymentSpec': {
                        'executors': {
                            'exec-comp': {
                                'podMetadata': {
                                    'labels': {
                                        'kubeflow.com/kfp': 'pipeline-node'
                                    }
                                }
                            }
                        }
                    }
                }
            }
        }

    def test_add_same_one(self):

        @dsl.pipeline
        def my_pipeline():
            task = comp()
            kubernetes.add_pod_label(
                task,
                label_key='kubeflow.com/kfp',
                label_value='pipeline-node',
            )
            kubernetes.add_pod_label(
                task,
                label_key='kubeflow.com/kfp',
                label_value='pipeline-node2',
            )

        assert json_format.MessageToDict(my_pipeline.platform_spec) == {
            'platforms': {
                'kubernetes': {
                    'deploymentSpec': {
                        'executors': {
                            'exec-comp': {
                                'podMetadata': {
                                    'labels': {
                                        'kubeflow.com/kfp': 'pipeline-node2'
                                    }
                                }
                            }
                        }
                    }
                }
            }
        }

    def test_add_two_and_mix(self):

        @dsl.pipeline
        def my_pipeline():
            task = comp()
            kubernetes.add_pod_label(
                task,
                label_key='kubeflow.com/kfp',
                label_value='pipeline-node',
            )
            kubernetes.add_pod_label(
                task,
                label_key='kubeflow.com/common',
                label_value='test',
            )
            kubernetes.add_pod_annotation(
                task,
                annotation_key='run_id',
                annotation_value='123456',
            )
            kubernetes.add_pod_annotation(
                task,
                annotation_key='experiment_id',
                annotation_value='234567',
            )

        assert json_format.MessageToDict(my_pipeline.platform_spec) == {
            'platforms': {
                'kubernetes': {
                    'deploymentSpec': {
                        'executors': {
                            'exec-comp': {
                                'podMetadata': {
                                    'annotations': {
                                        'run_id': '123456',
                                        'experiment_id': '234567'
                                    },
                                    'labels': {
                                        'kubeflow.com/kfp': 'pipeline-node',
                                        'kubeflow.com/common': 'test'
                                    }
                                }
                            }
                        }
                    }
                }
            }
        }

    def test_respects_other_configuration(self):

        @dsl.pipeline
        def my_pipeline():
            task = comp()
            kubernetes.use_secret_as_volume(
                task, secret_name='my-secret', mount_path='/mnt/my_vol')
            kubernetes.add_pod_annotation(
                task,
                annotation_key='run_id',
                annotation_value='123456',
            )

        assert json_format.MessageToDict(my_pipeline.platform_spec) == {
            'platforms': {
                'kubernetes': {
                    'deploymentSpec': {
                        'executors': {
                            'exec-comp': {
                                'podMetadata': {
                                    'annotations': {
                                        'run_id': '123456'
                                    }
                                },
                                'secretAsVolume': [{
                                    'secretName': 'my-secret',
                                    'mountPath': '/mnt/my_vol',
                                    'optional': False
                                }]
                            }
                        }
                    }
                }
            }
        }


@dsl.component
def comp():
    pass



# Copyright 2024 The Kubeflow Authors
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from google.protobuf import json_format
from kfp.dsl import PipelineTask
from kfp.kubernetes import common


def set_timeout(
    task: PipelineTask,
    seconds: int,
) -> PipelineTask:
    """Add timeout to the task Pod's `active_deadline_seconds
    <https://kubernetes.io/docs/reference/generated/kubernetes-api/v1.26/#podspec-v1-core>`_.

    Timeout an integer greater than 0, corresponding to the podspec active_deadline_seconds <https://kubernetes.io/docs/reference/generated/kubernetes-api/v1.26/#podspec-v1-core`_ field.
    Integer 0 means removing the timeout fields from previous functions.

    Args:
        task: Pipeline task.
        seconds: Value of the active_deadline_seconds.

    Returns:
        Task object with an updated active_deadline_seconds.
    """

    msg = common.get_existing_kubernetes_config_as_message(task)
    if seconds >= 0:
        msg.active_deadline_seconds = seconds
    else:
        raise ValueError(
            f'Argument for "seconds" must be an integer greater or equals to 0. Got invalid input: {seconds}. '
        )
    task.platform_config['kubernetes'] = json_format.MessageToDict(msg)

    return task
